@@ -9,7 +9,6 @@ Tests for the Spyder kernel
 """
 
 import os
-
 import pytest
 
 from spyder.config.main import CONF
@@ -18,6 +17,23 @@ from spyder.utils.encoding import to_fs_from_unicode
 from spyder.utils.ipython.kernelspec import SpyderKernelSpec
 
 
+def test_python_interpreter(tmpdir):
+    """Test the validation of the python interpreter."""
+    # Set a non existing python interpreter
+    interpreter = str(tmpdir.mkdir('interpreter').join('python'))
+    CONF.set('main_interpreter', 'default', False)
+    CONF.set('main_interpreter', 'custom', True)
+    CONF.set('main_interpreter', 'executable', interpreter)
+
+    # Create a kernel spec
+    kernel_spec = SpyderKernelSpec()
+
+    # Assert that the python interprerter is the default one
+    assert interpreter not in kernel_spec.argv
+    assert CONF.get('main_interpreter', 'default')
+    assert not CONF.get('main_interpreter', 'custom')
+
+
 @pytest.mark.skipif(os.name != 'nt' or not PY2,
                     reason="It only makes sense on Windows and Python 2")
 def test_env_vars():
