@@ -24,6 +24,10 @@ from qtpy.QtWidgets import (QApplication, QCheckBox, QDialogButtonBox, QDialog,
                             QHeaderView)
 
 from pandas import DataFrame, DatetimeIndex, Series
+try:
+    from pandas._libs.tslib import OutOfBoundsDatetime
+except ImportError:  # For pandas version < 0.20
+    from pandas.tslib import OutOfBoundsDatetime
 import numpy as np
 
 # Local imports
@@ -42,7 +46,7 @@ from spyder.widgets.variableexplorer.arrayeditor import get_idx_rect
 REAL_NUMBER_TYPES = (float, int, np.int64, np.int32)
 COMPLEX_NUMBER_TYPES = (complex, np.complex64, np.complex128)
 # Used to convert bool intrance to false since bool('False') will return True
-_bool_false = ['false', '0']
+_bool_false = ['false', 'f', '0', '0.', '0.0', ' ']
 
 # Default format for data frames with floats
 DEFAULT_FORMAT = '%.6g'
@@ -258,6 +262,8 @@ class DataFrameModel(QAbstractTableModel):
         # handling, so fallback uses iloc
         try:
             value = self.df.iat[row, column]
+        except OutOfBoundsDatetime:
+            value = self.df.iloc[:, column].astype(str).iat[row]
         except:
             value = self.df.iloc[row, column]
         return value
@@ -315,6 +321,14 @@ class DataFrameModel(QAbstractTableModel):
                     self.df.sort(columns=self.df.columns[column-1],
                                  ascending=ascending, inplace=True,
                                  kind='mergesort')
+                except ValueError as e:
+                    # Not possible to sort on duplicate columns #5225
+                    QMessageBox.critical(self.dialog, "Error",
+                                         "ValueError: %s" % to_text_string(e))
+                except SystemError as e:
+                    # Not possible to sort on category dtypes #5361
+                    QMessageBox.critical(self.dialog, "Error",
+                                         "SystemError: %s" % to_text_string(e))
                 self.update_df_index()
             else:
                 self.df.sort_index(inplace=True, ascending=ascending)
@@ -351,21 +365,21 @@ class DataFrameModel(QAbstractTableModel):
         else:
             val = from_qvariant(value, str)
             current_value = self.get_value(row, column-1)
-            if isinstance(current_value, bool):
+            if isinstance(current_value, (bool, np.bool_)):
                 val = bool_false_check(val)
-            supported_types = (bool,) + REAL_NUMBER_TYPES + COMPLEX_NUMBER_TYPES
-            if (isinstance(current_value, supported_types) or 
+            supported_types = (bool, np.bool_) + REAL_NUMBER_TYPES
+            if (isinstance(current_value, supported_types) or
                     is_text_string(current_value)):
                 try:
                     self.df.iloc[row, column-1] = current_value.__class__(val)
-                except ValueError as e:
+                except (ValueError, OverflowError) as e:
                     QMessageBox.critical(self.dialog, "Error",
-                                         "Value error: %s" % str(e))
+                                         str(type(e).__name__) + ": " + str(e))
                     return False
             else:
                 QMessageBox.critical(self.dialog, "Error",
-                                     "The type of the cell is not a supported "
-                                     "type")
+                                     "Editing dtype {0!s} not yet supported."
+                                     .format(type(current_value).__name__))
                 return False
         self.max_min_col_update()
         return True
